{******************************************************************************
 *                                                                            *
 *                                LayGO (tm)                                  *
 *           Copyright (c) 1998-2000 by Advanced Relay Corporation            *
 *                           All rights reserved.                             *
 *                                                                            *
 ******************************************************************************

  $Source: q:/laygorcs/Vtx.pas $

      $Id: Vtx.pas 1.2 2000/02/29 01:17:15 tiktin Release $

  EXTERNAL MODULE DESCRIPTION

    Exports the Verified Transmission (Vtx) routines.

 ******************************************************************************}

unit Vtx;

interface

uses
    SysUtils, Hex;

const
    ERROR_INVALID_BUFFER_SIZE = -1;
    ERROR_INVALID_VTX_REFERENCE = -2;
    ERROR_SEQUENCE_NUMBER_DISCREPANCY = -3;
    ERROR_SIZE_DISCREPANCY = -4;
    ERROR_PATTERN_TYPE_DISCREPANCY = -5;
    ERROR_DATA_DISCREPANCY = -6;
    ERROR_NONE = 0;

    MODULUS = 7;
    SIZEOF_HEADER = 2;
    BUFFER_MIN = 2;
    BUFFER_MAX = 4096;
    PAD = 4;
    DIRECTION_BIT = 1;
    SEQUENCE_BITS = $E;
    SIZE_LOW_BITS = $F0;
    SIZE_HIGH_BITS = $FF;
    SIZE_LOW_NIBBLE = $0F;

procedure VtxInitialize(max: Cardinal);
function  VtxErrorMessage(i: Integer): String;
function  VtxValidateRx(buffer: HexPtr; size: Cardinal): Integer;
function  VtxGetTx(size: Cardinal): HexPtr;
procedure VtxNextTx();
procedure VtxReset();
procedure VtxResetRx();
procedure VtxResetTx();

implementation

var
    VtxMax:         Cardinal;
    VtxForward:     HexPtr;
    VtxReverse:     HexPtr;
    VtxTxSequence:  Cardinal;
    VtxTxForward:   Boolean;
    VtxRxSequence:  Cardinal;

procedure VtxInitialize(max: Cardinal);
begin
    if max < BUFFER_MIN then
        VtxMax := BUFFER_MIN
    else if max > BUFFER_MAX then
        VtxMax := BUFFER_MAX
    else
        VtxMax := max;

    VtxForward := AllocMem(VtxMax + PAD);
    VtxReverse := AllocMem(VtxMax + PAD);

    HexFill(VtxForward, SIZEOF_HEADER, VtxMax, True);
    HexFill(VtxReverse, SIZEOF_HEADER, VtxMax, False);

    VtxTxSequence   := 0;
    VtxRxSequence   := 0;
    VtxTxForward    := True;
end;

function VtxGetTx(size: Cardinal): HexPtr;
var
    buffer: HexPtr;
    hptr:   HexPtr;
    bit:    Cardinal;
begin
    if VtxTxForward then
    begin
        buffer  := VtxForward;
        bit     := 0;
    end
    else
    begin
        buffer  := VtxReverse;
        bit     := 1;
    end;

    hptr    := buffer;
    hptr^   := Byte((((size - 1) and SIZE_LOW_NIBBLE) shl 4) or
                    (((VtxTxSequence shl 1) and SEQUENCE_BITS) or bit));
    hptr    := Ptr(Integer(hptr) + 1);
    hptr^   := Byte(((size - 1) shr 4) and SIZE_HIGH_BITS);

    VtxGetTx := buffer;
end;

procedure VtxNextTx();
begin
    VtxTxSequence   := (VtxTxSequence + 1) mod MODULUS;
    VtxTxForward    := Not VtxTxForward;
end;

procedure VtxReset();
begin
    VtxResetRx();
    VtxResetTx();
end;

procedure VtxResetRx();
begin
    VtxRxSequence := 0;
end;

procedure VtxResetTx();
begin
    VtxTxSequence   := 0;
    VtxTxForward    := True;
end;

function VtxValidateRx(buffer: HexPtr; size: Cardinal): Integer;
var
    rxSequence: Cardinal;
    rxSize:     Cardinal;
    pattern:    HexPtr;
    i:          Cardinal;
    p0:         HexPtr;
    p1:         HexPtr;
begin

    VtxValidateRx := ERROR_NONE;

    p0  := buffer;
    p1  := Ptr(Integer(buffer) + 1);

    if (size < BUFFER_MIN) or (size > VtxMax) then
        VtxValidateRx := ERROR_INVALID_BUFFER_SIZE
    else
    begin
        rxSequence := (Cardinal(p0^) and SEQUENCE_BITS) shr 1;

        if rxSequence <> VtxRxSequence then
        begin
            VtxRxSequence := (rxSequence + 1) mod MODULUS;
            VtxValidateRx := ERROR_SEQUENCE_NUMBER_DISCREPANCY;
        end
        else
            VtxRxSequence := (VtxRxSequence + 1) mod MODULUS;

        rxSize := (Cardinal(p1^) shl 4) + ((Cardinal(p0^) and SIZE_LOW_BITS) shr 4) + 1;

        if rxSize <> size then
            VtxValidateRx := ERROR_SIZE_DISCREPANCY;

        if (Cardinal(p0^) and 1) <> 0 then
            pattern := VtxReverse
        else
            pattern := VtxForward;

        For i := SIZEOF_HEADER to size - 1 do
        begin
            if buffer^ <> pattern^ then
            begin
                VtxValidateRx := ERROR_DATA_DISCREPANCY;
                Break;
            end
            else
            begin
                Inc(buffer);
                Inc(pattern);
            end;
        end
    end;
end;

function VtxErrorMessage(i: Integer): String;
begin
    Case i of
    ERROR_INVALID_BUFFER_SIZE:
        VtxErrorMessage := 'Invalid buffer size';
    ERROR_SEQUENCE_NUMBER_DISCREPANCY:
        VtxErrorMessage := 'Sequence number discrepancy';
    ERROR_SIZE_DISCREPANCY:
        VtxErrorMessage := 'Size discrepancy';
    ERROR_DATA_DISCREPANCY:
        VtxErrorMessage := 'Data discrepancy';
    else
        VtxErrorMessage := 'Unknown error';
    end;
end;

{******************************************************************************
                                 End of File
 ******************************************************************************}
end.
